/* digital oszilloskop mit avr
 by: Clemens Helfmeier
 25.8.06
 */


#include "commandline.h"


/* this file contains the commandline interface for the Autopilot via serial line */

#include <avr/wdt.h>
#include <avr/delay.h>
#define reset()		{ wdt_enable(WDTO_30MS); }

#include <stdlib.h>

#define LINE_LEN		80
char line[LINE_LEN];
u8 line_index = 0;

#define LINE_READY		(1<<0)
u8 status = 0;

#define CMDOUT_DBG      (0)
#define CMDOUT_PILOT    (1)
#define CMDOUT_INPUT    (2)
#define CMDOUT_ADIS     (3)
#define CMDOUT_NMEA     (4)
u8 cmd_output = 0;

u8 cmd_timeout = 10;
u8 cmd_timer = 1;

void cmd_input( char c ) {
	// takes the newly recieved character
	if ((status & LINE_READY) != 0)
		return;
	if ((c == '\b') && (line_index > 0)) {
		line_index--;
		line[line_index] = 0;
	} else if (c == '\n') {
		status |= LINE_READY;
		line[line_index] = 0;
	} else if (line_index < LINE_LEN) {
		line[line_index++] = c;
	}
}

u8 cmd_cmp( char * s, char * t, u8 l ) {
	// compares the bufferst at s and t for length l
	while (l-- > 0)
		if (*s++ != *t++)
			return 1;
	return 0;
}


u8 cmd_checkdisplay( void ) {
	/* display the things every 10 second */
	/*
	if (timer1 == commit_timer) {
		commit_timer = timer1 + 5;
		return 1;
	}
	*/
	return (cmd_timer == 0);
}

void cmd_display( void ) {
	/* display the status or whatever has to be displayed */
	cmd_sends( itoa( timer1, s, 10 ) );
	cmd_sendc( '.');
	cmd_sends( itoa( timer100, s, 10 ) );
	cmd_timer = cmd_timeout;

	oszi_dbg( 0 );
	cmd_dbg( 0 );
	main_dbg( 0 );
}

void cmd_main( void ) {

	// should be called regularily
	if (status & LINE_READY) {
	    // the user sent a line, process it.
	    // write a timestamp
	    if (cmd_output == CMDOUT_DBG) {
		cmd_sends( itoa( timer1, s, 10 ) );
		cmd_sendc( '.');
		cmd_sends( itoa( timer100, s, 10 ) );
		cmd_sendc( '\n' );
	    }
	    char *str = line;
	    str = cmd_skip_spaces(str);
	    if (cmd_cmp(str, "oszi", 4) == 0) {
			oszi_dbg( str+4 );
	    } else if (cmd_cmp(str, "reset", 5) == 0) {
			/* perform a reset */
			reset();
	    } else if (cmd_cmp(str, "cmd", 3) == 0) {
			cmd_dbg( str+3 );
		} else if (cmd_cmp(str, "main", 4) == 0) {
			main_dbg( str+4 );
		}

	    // line processed,reset input buffer
	    memset( line, 0, LINE_LEN );
	    line_index = 0;
	    status &= ~LINE_READY;
	    if (cmd_output == CMDOUT_DBG) {
		// new prompt
		cmd_sendcs( "\n> " );
	    }
	}


	// is there something to display?
	if ((cmd_checkdisplay() != 0) && (cmd_output == CMDOUT_DBG)){
		/* clear the command line */
		for (u8 i = line_index + 2; i>0; i--)
			cmd_sendc( '\b' );

		/* output the data ... */
		cmd_display();

		/* re-send the command line */
		cmd_sendcs( "\n> " );
		cmd_sends( line );

	}

}

void cmd_init() {
	// initialize the command line
    cmd_sendcs( "command line interface ready.\n> " );
    // load default data
    cmd_output = 0;
}

char *cmd_skip_spaces( char *str ) {
	while (*str == ' ') str++;
	return str;
}

void cmd_timer1s( void ) {
	if (cmd_timer != 0) cmd_timer--;
}

void cmd_dbg( char* str ) {
	if (str == 0) {
	} else {
		str = cmd_skip_spaces(str);
		if (cmd_cmp(str, "set", 3) == 0) {
			str = cmd_skip_spaces(str+3);
			if (cmd_cmp(str, "timeout", 7) == 0) {
			    cmd_timeout = atoi(str+7);
			} else if (cmd_cmp(str, "output", 6) == 0) {
			    cmd_output = atoi(str+6);
			}
		}
	}
}

// alle Debug-Outputs werden nur gesendet wenn cmd_output auf dem jeweiligen
// File steht.
void dbg_oszis( char *str ) { if (cmd_output == CMDOUT_OSZI) cmd_sends( str ); }
void dbg_oszics_( char *str ) { if (cmd_output == CMDOUT_OSZI) cmd_sendcs_( str ); }
void dbg_oszic( char c ) { if (cmd_output == CMDOUT_OSZI) cmd_sendc( c ); }


