/* digital oszilloskop mit avr
 by: Clemens Helfmeier
 25.8.06
 */


#include "board.h"
#include "uart.h"
#include "commandline.h"
#include "oszi.h"
#include <avr/pgmspace.h>
#include <avr/signal.h>
#include <util/delay.h>

char s[10];

u8 timer1;
u8 timer100;
		

int main( void ) {

	for (u16 i = 0; i<1000; i++) _delay_ms(1);

	// initialize the timer interrupt
	TCCR0 = (1<<WGM01) | (0<<WGM00) | (1<<CS02) | (0<<CS01) | (1<<CS00);
	OCR0 = 144;             // this results in 14745600 / 1024 / 144 = 100Hz
	TIMSK |= (1<<OCF0);

	uart_init();
	cmd_dbg( "uart_init complete.\n" );
	
	cmd_init();
	cmd_dbg( "cmd_init complete.\n" );

	oszi_init();
	cmd_dbg( "oszi_init complete.\n" );

	sei();


	// CTC, ICR, OC1A is non-inverted output
	TCCR1A = (1<<WGM11) | (0<<WGM10) | (1<<COM1A1) | (0<<COM1A0);
	TCCR1B = (1<<WGM13) | (1<<WGM12) | (0<<CS12) | (0<<CS11) | (1<<CS10);
	DDRD |= (1<<5);
	ICR1 = 32;	// 1Mhz PWM output, 4 bit
	OCR1A = 8;  // 50% output

	// clock modulator
	DDRD |= (1<<PD7);
	PORTD |= (1<<PD7);
	
	while (1) {
		cmd_main();
		oszi_main();
	}
	
}

void main_dbg( char *str ) {
	if (str == 0) {
	} else {
		str = cmd_skip_spaces(str);
		if (cmd_cmp( str, "set", 3 ) == 0) {
			str = cmd_skip_spaces(str+3);
			if (cmd_cmp( str, "icr", 3) == 0) {
				ICR1 = atoi( str+3 );
			} else if (cmd_cmp( str, "ocr", 3 ) == 0) {
				OCR1A = atoi( str+3 );
			}
		}
	}
}

void uart0_rx_handler( char c ) {
	cmd_input( c );
}


ISR(TIMER0_COMP_vect) {
	if (timer100-- <= 0) {
		timer100 = 100;
		timer1++;
		cmd_timer1s();
	}
}
