

/** @file uart.c
 * UART routines for ATMega162. Supports dual UART
 *
 * @author Clemens Helfmeier
 * @date 09.07.2006
 */

#include "board.h"
#include "uart.h"
#include <avr/pgmspace.h>
#include <avr/signal.h>


#ifdef ONLY_ONE_UART
	#define UCSR0A UCSRA
	#define UCSR0B UCSRB
	#define UCSR0C UCSRC
	#define UBRR0L UBRRL
	#define UBRR0H UBRRH
	#define UDR0 UDR
	#define TXEN0 TXEN
	#define RXEN0 RXEN
	#define RXCIE0 RXCIE
	#define URSEL0 URSEL
	#define UMSEL0 UMSEL
	#define UPM01 UPM1
	#define UPM00 UPM0
	#define USBS0 USBS
	#define UCSZ01 UCSZ1
	#define UCSZ00 UCSZ0
	#define UDRE0 UDRE
	#define RXC0 RXC
	#define USART0_RXC_vect USART_RXC_vect
#endif

void uart_init( void ) {
	/* initialize both uarts. */
	// configure io-pins
#ifdef UART1
	DDRB = (DDRB & (~(1<<PB2))) | (1<<PB3);
	// configure transmission format
	UCSR1A = 0;
#ifdef UART1_RXHANDLER
	UCSR1B = (1<<TXEN1) | (1<<RXEN1) | (1<<RXCIE1);
#else
	UCSR1B = (1<<TXEN1) | (1<<RXEN1);
#endif
	UCSR1C = (1<<URSEL1) | (0<<UMSEL1) | (0<<UPM11) | (0<<UPM10) | (0<<USBS1) | (1<<UCSZ11) | (1<<UCSZ10);
	// configure baud rate
	UBRR1L = UBRR1 & 0xFF;
	UBRR1H = (UBRR1 >> 8) & 0x0F;
#endif

#ifdef UART0
	DDRD = (DDRD & (~(1<<PD0))) | (1<<PD1);
	// configure transmission format
	UCSR0A = 0;
#ifdef UART0_RXHANDLER
	UCSR0B = (1<<TXEN0) | (1<<RXEN0) | (1<<RXCIE0);
#else
	UCSR0B = (1<<TXEN0) | (1<<RXEN0);
#endif
	UCSR0C = (1<<URSEL0) | (0<<UMSEL0) | (0<<UPM01) | (0<<UPM00) | (0<<USBS0) | (1<<UCSZ01) | (1<<UCSZ00);
	// configure baud rate
	UBRR0L = UBRR0 & 0xFF;
	UBRR0H = (UBRR0 >> 8) & 0x0F;
#endif

}

/** UART 0 Functions */
#ifdef UART0
void uart0_putc( char s ) {
	while ((UCSR0A & (1<<UDRE0)) == 0);
	/*
	if (s == '\n') {
		UDR0 = 13;
		while ((UCSR0A & (1<<UDRE0)) == 0);
		UDR0 = 10;
	} else
	*/
		UDR0 = s;
}
#ifndef UART0_RXHANDLER
char uart0_getc( void ) {
	while ((UCSR0A & (1<<RXC0)) != 0);
	return UDR0;
}
void uart0_gets( char * s, u8 maxlen ) {
}
#else
ISR(USART0_RXC_vect) {
	UART0_RXHANDLER
}
#endif

void uart0_puts_p( char * s ) {
	while (pgm_read_byte(s) != 0)
		uart0_putc( pgm_read_byte(s++) );
}
void uart0_puts( char * s ) {
	while (*s != 0)
		uart0_putc( *s++ );
}

#endif

/** UART 1 Functions */
#ifdef UART1
void uart1_putc( char s ) {
	while ((UCSR1A & (1<<UDRE1)) == 0);
	/*
	if (s == '\n') {
	UDR1 = 13;
	while ((UCSR1A & (1<<UDRE1)) == 0);
	UDR1 = 10;
} else
	*/
	UDR1 = s;
}
#ifndef UART1_RXHANDLER
char uart1_getc( void ) {
	while ((UCSR1A & (1<<RXC1)) != 0);
	return UDR1;
}
void uart1_gets( char * s, u8 maxlen ) {
}
#else
ISR(USART1_RXC_vect) {
	UART1_RXHANDLER
}
#endif
void uart1_puts_p( char * s ) {
	while (pgm_read_byte(s) != 0)
		uart1_putc( pgm_read_byte(s++) );
}

void uart1_puts( char * s ) {
	while (*s != 0)
		uart1_putc( *s++ );
}
#endif

